@@ -1,12 +1,12 @@
-mod file;
+mod file_ext;
 mod page;
 mod page_no;
 mod size_info;
 
+pub use page::Page;
 pub use page_no::*;
 
-use file::FileExt;
-use page::Page;
+use file_ext::FileExt;
 use size_info::SizeInfo;
 use std::fs::File;
 use std::io::{Error, ErrorKind, Result};
@@ -16,25 +16,28 @@ use std::sync::atomic::{AtomicU32, Ordering};
 use page_lock::PageLocker;
 use tokio::task::spawn_blocking;
 
+/// **NOTE**: Caller must ensure that closure must not panic.
+/// If it does, it will be undefined behavior.
+macro_rules! into_async { [$body:expr] => { unsafe { spawn_blocking(move || $body).await.unwrap_unchecked() } }; }
 macro_rules! err { [$cond: expr, $msg: expr] => { if $cond { return Err(Error::new(ErrorKind::InvalidInput, $msg)); } }; }
 
-pub struct Pages<K: PageNo, const NBYTES: usize> {
+pub struct Pages<N, const SIZE: usize> {
     /// Total Page number
     len: AtomicU32,
     file: &'static File,
-    locker: PageLocker<K>,
+    locker: PageLocker<N>,
     size_info: SizeInfo,
 }
 
-impl<K: PageNo, const NBYTES: usize> Pages<K, NBYTES> {
+impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
     /// Create a new `Pages` instance.
     pub fn open(path: impl AsRef<Path>) -> Result<Self> {
-        err!(NBYTES < 64, "Page size should >= 64 bytes");
-        err!(NBYTES > 1024 * 256, "Page size should > 256 kilobytes");
+        err!(SIZE < 64, "Page size should >= 64 bytes");
+        err!(SIZE > 1024 * 256, "Page size should > 256 kilobytes");
 
         let size_info = SizeInfo {
-            block_size: NBYTES as u32,
-            pages_len_nbytes: K::SIZE as u8,
+            block_size: SIZE as u32,
+            pages_len_nbytes: N::SIZE as u8,
         };
         let file = File::options()
             .read(true)
@@ -44,17 +47,15 @@ impl<K: PageNo, const NBYTES: usize> Pages<K, NBYTES> {
 
         let file_len = file.metadata()?.len();
         // So that, There is no residue bytes.
-        if file_len % NBYTES as u64 != 0 {
+        if file_len % SIZE as u64 != 0 {
             return Err(ErrorKind::InvalidData.into());
         }
-        let mut len = file_len as u32 / NBYTES as u32;
-        // Is new file? set the length to 1.
-        if file_len == 0 {
-            len = 1;
-            file.set_len(NBYTES as u64)?;
-        }
-        // Exist File?  Check size info.
-        else {
+        let len = if file_len == 0 {
+            // Is new file? set the length to 1.
+            file.set_len(SIZE as u64)?;
+            1
+        } else {
+            // Exist File?  Check size info.
             let mut buf = [0; 4];
             file.read_exact_at(&mut buf, 0)?;
             let info = SizeInfo::from(buf);
@@ -62,53 +63,39 @@ impl<K: PageNo, const NBYTES: usize> Pages<K, NBYTES> {
                 info != size_info,
                 format!("Expected {:?}, but got: {:?}", info, size_info)
             );
-        }
-        let len = AtomicU32::new(len);
-        let file: &'static File = Box::leak(Box::new(file));
+            file_len as u32 / SIZE as u32
+        };
         Ok(Self {
             size_info,
-            file,
+            len: AtomicU32::new(len),
+            file: Box::leak(Box::new(file)),
             locker: PageLocker::new(),
-            len,
         })
     }
 
     pub async fn sync_data(&self) -> Result<()> {
         let file = self.file;
-        unsafe {
-            spawn_blocking(move || file.sync_data())
-                .await
-                .unwrap_unchecked()
-        }
+        into_async!(file.sync_data())
     }
 
     pub async fn sync_all(&self) -> Result<()> {
         let file = self.file;
-        unsafe {
-            spawn_blocking(move || file.sync_all())
-                .await
-                .unwrap_unchecked()
-        }
+        into_async!(file.sync_all())
     }
 
-    pub async fn read(&self, num: K) -> Result<[u8; NBYTES]> {
+    pub async fn read(&self, num: N) -> Result<[u8; SIZE]> {
         debug_assert!(num.as_u32() < self.len());
-
         self.locker.unlock(num).await;
         let num = num.as_u32() as u64;
         let file = self.file;
-        unsafe {
-            spawn_blocking(move || {
-                let mut buf = [0; NBYTES];
-                file.read_exact_at(&mut buf, NBYTES as u64 * num)?;
-                Ok(buf)
-            })
-            .await
-            .unwrap_unchecked()
-        }
+        into_async!({
+            let mut buf = [0; SIZE];
+            file.read_exact_at(&mut buf, SIZE as u64 * num)?;
+            Ok(buf)
+        })
     }
 
-    pub async fn goto(&self, num: K) -> Result<Page<'_, K, NBYTES>> {
+    pub async fn goto(&self, num: N) -> Result<Page<'_, N, SIZE>> {
         let buf = self.read(num).await?;
         let _lock = self.locker.lock(num);
         Ok(Page {
@@ -119,35 +106,31 @@ impl<K: PageNo, const NBYTES: usize> Pages<K, NBYTES> {
         })
     }
 
-    pub async fn write(&self, num: K, buf: [u8; NBYTES]) -> Result<()> {
+    pub async fn write(&self, num: N, buf: [u8; SIZE]) -> Result<()> {
         debug_assert!(num.as_u32() < self.len());
-
         let num = num.as_u32() as u64;
         let file = self.file;
-        unsafe {
-            spawn_blocking(move || file.write_all_at(&buf, NBYTES as u64 * num))
-                .await
-                .unwrap_unchecked()
-        }
+        into_async!(file.write_all_at(&buf, SIZE as u64 * num))
     }
 
     /// Increase page number by `count`
-    pub async fn alloc(&self, count: u32) -> Result<u32> {
+    pub async fn alloc(&self, count: u32) -> Result<N> {
         let old_len = self.len.fetch_add(count, Ordering::SeqCst);
         let file = self.file;
-        unsafe {
-            spawn_blocking(move || {
-                file.set_len(NBYTES as u64 * (old_len + count) as u64)?;
-                Ok(old_len)
-            })
-            .await
-            .unwrap_unchecked()
-        }
+        into_async!(file.set_len(SIZE as u64 * (old_len + count) as u64))?;
+        Ok(N::new(old_len))
     }
 
-    pub async fn create(&self, buf: [u8; NBYTES]) -> Result<()> {
+    pub async fn create(&self, buf: [u8; SIZE]) -> Result<N> {
         let num = self.len.fetch_add(1, Ordering::SeqCst);
-        self.write(PageNo::new(num), buf).await
+        self.write(PageNo::new(num), buf).await?;
+        Ok(N::new(num))
+    }
+
+    pub fn alloc_sync(&self, count: u32) -> Result<N> {
+        let old_len = self.len.fetch_add(count, Ordering::SeqCst);
+        self.file.set_len(SIZE as u64 * (old_len + count) as u64)?;
+        Ok(N::new(old_len))
     }
 
     #[allow(clippy::len_without_is_empty)]
@@ -155,12 +138,12 @@ impl<K: PageNo, const NBYTES: usize> Pages<K, NBYTES> {
         self.len.load(Ordering::SeqCst)
     }
 
-    pub fn inner(&self) -> &'static File {
+    pub fn inner(&self) -> &File {
         self.file
     }
 }
 
-impl<K: PageNo, const NBYTES: usize> Drop for Pages<K, NBYTES> {
+impl<N, const SIZE: usize> Drop for Pages<N, SIZE> {
     fn drop(&mut self) {
         self.file
             .write_all_at(&self.size_info.to_bytes(), 0)
@@ -173,7 +156,7 @@ mod tests {
     use super::*;
 
     async fn read_write() -> Result<()> {
-        let pages = Pages::<u16, 64>::open("test.db")?;
+        let pages = Pages::<u16, 64>::open("test1.db")?;
 
         assert_eq!(pages.len(), 1);
         pages.create([0; 64]).await?;
@@ -184,13 +167,14 @@ mod tests {
         page.write().await?;
 
         assert_eq!(pages.read(1).await?, [1; 64]);
-        pages.create([0; 64]).await?;
+        pages.alloc(1).await?;
 
         assert_eq!(pages.len(), 3);
 
         pages.write(2, [2; 64]).await?;
         assert_eq!(pages.read(2).await?, [2; 64]);
-        std::fs::remove_file("test.db")
+
+        std::fs::remove_file("test1.db")
     }
 
     async fn alloc_page() -> Result<()> {
