@@ -1,7 +1,8 @@
 use std::convert::TryInto;
+use std::fmt::Debug;
 use std::hash::Hash;
 
-pub trait PageNo: Eq + Hash + Copy + Unpin + Send + Sync + Ord + Default {
+pub trait PageNo: Ord + Hash + Copy + Unpin + Sync + Send + Default + Debug {
     /// Total number of bytes.
     const SIZE: usize;
     fn new(_: u32) -> Self;
@@ -28,7 +29,7 @@ macro_rules! impl_trait {
 impl_trait!(PageNo for u8:1 u16:2 u32:4);
 
 #[repr(transparent)]
-#[derive(Hash, PartialEq, Eq, Clone, Copy, Debug, PartialOrd, Ord, Default)]
+#[derive(Hash, PartialEq, Eq, Clone, Copy, PartialOrd, Ord, Default, Debug)]
 pub struct U24(u32);
 
 impl PageNo for U24 {
